window.Gleam.initSearch([{"doc":"glv8","title":"glv8","content":"# glv8\n\nEver wanted to write functional type-safe postgresql procedures ? Let&#39;s use [gleam](https://gleam.run) !\n\n[![Package Version](https://img.shields.io/hexpm/v/glv8)](https://hex.pm/packages/glv8)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/glv8/)\n\nglv8 coworks with [plv8](https://plv8.github.io). It provides plv8&#39;s API bindings so one can\nwrite procedures with gleam and let `plv8` do the needed proxying. Specifically, gleam codes are\ncompiled as JavaScript and interoperate with plv8 ecosystem. For example\n\n```gleam\npub fn catch_sql_error() -&gt; Nil {\n  let r = database.execute(&quot;throw SQL error&quot;, Nil)\n  use &lt;- bool.guard(result.is_ok(r), Nil)\n\n  case r {\n    Error(glv8.DBErrorJson(j)) -&gt; elog_notice(j |&gt; json.to_string)\n    _ -&gt; elog_notice(&quot;should not come here&quot;)\n  }\n}\n```\n\nThe snippet can be provisioned with `plv8` declaration\n\n```sql\nCREATE FUNCTION catch_sql_error() RETURNS void AS\n$$\n  glv8.catch_sql_error();\n$$ LANGUAGE plv8;\n```\n\nMore examples can be found at `src/app/plv8.gleam`\nFurther documentation can be found at &lt;https://hexdocs.pm/glv8&gt;.\n\n## Development\n\n- write any gleam functions, with `pub fn` specifier in any packages.\n- provide needed proxying sqls\n- declare them in the `exports` and `sqls` of `bundle.gleam`\n- build (or watch build) them with npm scripts\n- the artifacts are `dist/glv8_init.sql` which wraps up all the exported gleam functions\n  and the needed proxying sqls\n- by default, the gleam functions use prefix `glv8.` as namespace, one can overwrite\n  it with envionment variable `GLV8_NAMESPACE`\n\n```sh\n$ npm run build\n$ npm run watch\n$ npm run clean\n$ npm run purge\n```\n\n## Deployment\n\n- postgresql database with `plv8` trusted language extension.\n- glv8 needs `SET plv8.start_proc = &#39;glv8_init&#39;;` either in `postgresql.conf` or session pre-request\n- deploy artifacts `glv8_init.sql` and all the other proxying sqls\n- enjoy\n\n## RoadMap\n\n- auto-generate proxying sqls as [plv8ify](https://github.com/divyenduz/plv8ify) with `glance` (it might be rigid and over-kill after all)\n","url":"index.html"},{"doc":"glv8","title":"DBError","content":"pub type DBError {\n  DBError(Nil)\n  DBErrorMessage(e: String)\n  DBErrorJson(j: Json)\n  DBErrorDecode(es: DecodeErrors)\n}\n\nDBError(Nil)\n\n\nDBErrorMessage(e: String)\n\n\nDBErrorJson(j: Json)\n\n\nDBErrorDecode(es: DecodeErrors)\n\n\nSynonyms:\nglv8.DBError\nglv8 DBError","url":"glv8.html#DBError"},{"doc":"glv8","title":"Function0","content":"pub type Function0(r) =\n  fn() -&gt; r\n\n\nSynonyms:\nglv8.Function0\nglv8 Function0","url":"glv8.html#Function0"},{"doc":"glv8","title":"Function1","content":"pub type Function1(a, r) =\n  fn(a) -&gt; r\n\n\nSynonyms:\nglv8.Function1\nglv8 Function1","url":"glv8.html#Function1"},{"doc":"glv8","title":"Function2","content":"pub type Function2(a, b, r) =\n  fn(a, b) -&gt; r\n\n\nSynonyms:\nglv8.Function2\nglv8 Function2","url":"glv8.html#Function2"},{"doc":"glv8","title":"Function3","content":"pub type Function3(a, b, c, r) =\n  fn(a, b, c) -&gt; r\n\n\nSynonyms:\nglv8.Function3\nglv8 Function3","url":"glv8.html#Function3"},{"doc":"glv8","title":"Function4","content":"pub type Function4(a, b, c, d, r) =\n  fn(a, b, c, d) -&gt; r\n\n\nSynonyms:\nglv8.Function4\nglv8 Function4","url":"glv8.html#Function4"},{"doc":"glv8","title":"JsObject","content":"pub type JsObject {\n  JsObject\n}\n\nJsObject\n\n\nSynonyms:\nglv8.JsObject\nglv8 JsObject","url":"glv8.html#JsObject"},{"doc":"glv8","title":"SqlFunction","content":"pub type SqlFunction(f) {\n  SqlFunction(f: f, sql: String)\n}\n\nSqlFunction(f: f, sql: String)\n\n\nSynonyms:\nglv8.SqlFunction\nglv8 SqlFunction","url":"glv8.html#SqlFunction"},{"doc":"glv8","title":"error_to_string","content":"pub fn error_to_string(e: DBError) -&gt; String\n\nSynonyms:\nglv8.error_to_string\nglv8 error_to_string","url":"glv8.html#error_to_string"},{"doc":"glv8","title":"glv8","content":"","url":"glv8.html"},{"doc":"glv8/database","title":"Cursor","content":"pub type Cursor\n\n\nSynonyms:\nglv8/database.Cursor\nglv8/database Cursor","url":"glv8/database.html#Cursor"},{"doc":"glv8/database","title":"PreparedPlan","content":"pub type PreparedPlan\n\n\nSynonyms:\nglv8/database.PreparedPlan\nglv8/database PreparedPlan","url":"glv8/database.html#PreparedPlan"},{"doc":"glv8/database","title":"Row","content":"pub type Row {\n  Row\n  NilRow(Nil)\n}\n\nRow\n\n\nNilRow(Nil)\n\n\nSynonyms:\nglv8/database.Row\nglv8/database Row","url":"glv8/database.html#Row"},{"doc":"glv8/database","title":"SubTransaction","content":"pub type SubTransaction =\n  fn() -&gt; Nil\n\n\nSynonyms:\nglv8/database.SubTransaction\nglv8/database SubTransaction","url":"glv8/database.html#SubTransaction"},{"doc":"glv8/database","title":"cursor_close","content":"pub fn cursor_close(cursor c: Cursor) -&gt; Nil\n\n\n\nSynonyms:\nglv8/database.cursor_close\nglv8/database cursor_close","url":"glv8/database.html#cursor_close"},{"doc":"glv8/database","title":"cursor_fetch","content":"pub fn cursor_fetch(cursor c: Cursor) -&gt; Result(Row, DBError)\n\n\n\nSynonyms:\nglv8/database.cursor_fetch\nglv8/database cursor_fetch","url":"glv8/database.html#cursor_fetch"},{"doc":"glv8/database","title":"cursor_fetch_as","content":"pub fn cursor_fetch_as(\n  cursor c: Cursor,\n  of f: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; Result(a, DBError)\n\n\n\nSynonyms:\nglv8/database.cursor_fetch_as\nglv8/database cursor_fetch_as","url":"glv8/database.html#cursor_fetch_as"},{"doc":"glv8/database","title":"cursor_fetch_rows","content":"pub fn cursor_fetch_rows(\n  cursor c: Cursor,\n  rows n: Int,\n) -&gt; Result(Array(Row), DBError)\n\n\n\nSynonyms:\nglv8/database.cursor_fetch_rows\nglv8/database cursor_fetch_rows","url":"glv8/database.html#cursor_fetch_rows"},{"doc":"glv8/database","title":"cursor_fetch_rows_as","content":"pub fn cursor_fetch_rows_as(\n  cursor c: Cursor,\n  rows n: Int,\n  of f: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; Result(Array(a), DBError)\n\n\n\nSynonyms:\nglv8/database.cursor_fetch_rows_as\nglv8/database cursor_fetch_rows_as","url":"glv8/database.html#cursor_fetch_rows_as"},{"doc":"glv8/database","title":"cursor_move","content":"pub fn cursor_move(cursor c: Cursor, rows n: Int) -&gt; Nil\n\n\n\nSynonyms:\nglv8/database.cursor_move\nglv8/database cursor_move","url":"glv8/database.html#cursor_move"},{"doc":"glv8/database","title":"decode","content":"pub fn decode(\n  rows rs: Array(Row),\n  decoder f: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; Result(Array(a), DBError)\n\n\n\nSynonyms:\nglv8/database.decode\nglv8/database decode","url":"glv8/database.html#decode"},{"doc":"glv8/database","title":"decode0","content":"pub fn decode0(\n  row r: Row,\n  decoder f: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; Result(a, DBError)\n\n\n\nSynonyms:\nglv8/database.decode0\nglv8/database decode0","url":"glv8/database.html#decode0"},{"doc":"glv8/database","title":"execute","content":"pub fn execute(\n  query q: String,\n  parameters p: a,\n) -&gt; Result(Array(Row), DBError)\n\n\n\nSynonyms:\nglv8/database.execute\nglv8/database execute","url":"glv8/database.html#execute"},{"doc":"glv8/database","title":"execute0","content":"pub fn execute0(\n  query q: String,\n  parameters p: a,\n) -&gt; Result(Int, DBError)\n\n\n\nSynonyms:\nglv8/database.execute0\nglv8/database execute0","url":"glv8/database.html#execute0"},{"doc":"glv8/database","title":"execute_as","content":"pub fn execute_as(\n  query q: String,\n  parameters p: a,\n  of f: fn(Dynamic) -&gt; Result(b, List(DecodeError)),\n) -&gt; Result(Array(b), DBError)\n\n\n\nSynonyms:\nglv8/database.execute_as\nglv8/database execute_as","url":"glv8/database.html#execute_as"},{"doc":"glv8/database","title":"plan_cursor","content":"pub fn plan_cursor(\n  plan pl: PreparedPlan,\n  parameters p: a,\n) -&gt; Cursor\n\n\n\nSynonyms:\nglv8/database.plan_cursor\nglv8/database plan_cursor","url":"glv8/database.html#plan_cursor"},{"doc":"glv8/database","title":"plan_execute","content":"pub fn plan_execute(\n  plan pl: PreparedPlan,\n  parameters p: a,\n) -&gt; Result(Array(Row), DBError)\n\n\n\nSynonyms:\nglv8/database.plan_execute\nglv8/database plan_execute","url":"glv8/database.html#plan_execute"},{"doc":"glv8/database","title":"plan_execute_as","content":"pub fn plan_execute_as(\n  plan pl: PreparedPlan,\n  parameters p: a,\n  of f: fn(Dynamic) -&gt; Result(b, List(DecodeError)),\n) -&gt; Result(Array(b), DBError)\n\n\n\nSynonyms:\nglv8/database.plan_execute_as\nglv8/database plan_execute_as","url":"glv8/database.html#plan_execute_as"},{"doc":"glv8/database","title":"plan_free","content":"pub fn plan_free(plan pl: PreparedPlan) -&gt; Nil\n\n\n\nSynonyms:\nglv8/database.plan_free\nglv8/database plan_free","url":"glv8/database.html#plan_free"},{"doc":"glv8/database","title":"prepare","content":"pub fn prepare(\n  query q: String,\n  paremeters p: Array(String),\n) -&gt; PreparedPlan\n\n\n\nSynonyms:\nglv8/database.prepare\nglv8/database prepare","url":"glv8/database.html#prepare"},{"doc":"glv8/database","title":"shift","content":"pub fn shift(\n  rs: Result(Array(a), DBError),\n) -&gt; Result(#(a, Array(a)), DBError)\n\n\n\nSynonyms:\nglv8/database.shift\nglv8/database shift","url":"glv8/database.html#shift"},{"doc":"glv8/database","title":"shift_as","content":"pub fn shift_as(\n  rs: Result(Array(a), DBError),\n  decoder f: fn(Dynamic) -&gt; Result(b, List(DecodeError)),\n) -&gt; Result(b, DBError)\n\n\n\nSynonyms:\nglv8/database.shift_as\nglv8/database shift_as","url":"glv8/database.html#shift_as"},{"doc":"glv8/database","title":"subtransaction","content":"pub fn subtransaction(\n  transaction t: fn() -&gt; Nil,\n) -&gt; Result(Nil, DBError)\n\n\n\nSynonyms:\nglv8/database.subtransaction\nglv8/database subtransaction","url":"glv8/database.html#subtransaction"},{"doc":"glv8/database","title":"glv8/database","content":"","url":"glv8/database.html"},{"doc":"glv8/util","title":"elog","content":"pub fn elog(level l: Symbol, message m: String) -&gt; Nil\n\n\n\nSynonyms:\nglv8/util.elog\nglv8/util elog","url":"glv8/util.html#elog"},{"doc":"glv8/util","title":"elog_debug1","content":"pub fn elog_debug1(message m: String) -&gt; Nil\n\n\n\nSynonyms:\nglv8/util.elog_debug1\nglv8/util elog_debug1","url":"glv8/util.html#elog_debug1"},{"doc":"glv8/util","title":"elog_debug2","content":"pub fn elog_debug2(message m: String) -&gt; Nil\n\n\n\nSynonyms:\nglv8/util.elog_debug2\nglv8/util elog_debug2","url":"glv8/util.html#elog_debug2"},{"doc":"glv8/util","title":"elog_debug3","content":"pub fn elog_debug3(message m: String) -&gt; Nil\n\n\n\nSynonyms:\nglv8/util.elog_debug3\nglv8/util elog_debug3","url":"glv8/util.html#elog_debug3"},{"doc":"glv8/util","title":"elog_debug4","content":"pub fn elog_debug4(message m: String) -&gt; Nil\n\n\n\nSynonyms:\nglv8/util.elog_debug4\nglv8/util elog_debug4","url":"glv8/util.html#elog_debug4"},{"doc":"glv8/util","title":"elog_debug5","content":"pub fn elog_debug5(message m: String) -&gt; Nil\n\n\n\nSynonyms:\nglv8/util.elog_debug5\nglv8/util elog_debug5","url":"glv8/util.html#elog_debug5"},{"doc":"glv8/util","title":"elog_error","content":"pub fn elog_error(message m: String) -&gt; Nil\n\n\n\nSynonyms:\nglv8/util.elog_error\nglv8/util elog_error","url":"glv8/util.html#elog_error"},{"doc":"glv8/util","title":"elog_info","content":"pub fn elog_info(message m: String) -&gt; Nil\n\n\n\nSynonyms:\nglv8/util.elog_info\nglv8/util elog_info","url":"glv8/util.html#elog_info"},{"doc":"glv8/util","title":"elog_log","content":"pub fn elog_log(message m: String) -&gt; Nil\n\n\n\nSynonyms:\nglv8/util.elog_log\nglv8/util elog_log","url":"glv8/util.html#elog_log"},{"doc":"glv8/util","title":"elog_notice","content":"pub fn elog_notice(message m: String) -&gt; Nil\n\n\n\nSynonyms:\nglv8/util.elog_notice\nglv8/util elog_notice","url":"glv8/util.html#elog_notice"},{"doc":"glv8/util","title":"elog_warning","content":"pub fn elog_warning(message m: String) -&gt; Nil\n\n\n\nSynonyms:\nglv8/util.elog_warning\nglv8/util elog_warning","url":"glv8/util.html#elog_warning"},{"doc":"glv8/util","title":"export","content":"pub fn export(o: JsObject, f: a) -&gt; JsObject\n\n\n\nSynonyms:\nglv8/util.export\nglv8/util export","url":"glv8/util.html#export"},{"doc":"glv8/util","title":"find_function0","content":"pub fn find_function0(function f: String) -&gt; fn() -&gt; a\n\n\n\nSynonyms:\nglv8/util.find_function0\nglv8/util find_function0","url":"glv8/util.html#find_function0"},{"doc":"glv8/util","title":"find_function1","content":"pub fn find_function1(function f: String) -&gt; fn(a) -&gt; b\n\n\n\nSynonyms:\nglv8/util.find_function1\nglv8/util find_function1","url":"glv8/util.html#find_function1"},{"doc":"glv8/util","title":"find_function2","content":"pub fn find_function2(function f: String) -&gt; fn(a, b) -&gt; c\n\n\n\nSynonyms:\nglv8/util.find_function2\nglv8/util find_function2","url":"glv8/util.html#find_function2"},{"doc":"glv8/util","title":"find_function3","content":"pub fn find_function3(function f: String) -&gt; fn(a, b, c) -&gt; d\n\n\n\nSynonyms:\nglv8/util.find_function3\nglv8/util find_function3","url":"glv8/util.html#find_function3"},{"doc":"glv8/util","title":"find_function4","content":"pub fn find_function4(function f: String) -&gt; fn(a, b, c, d) -&gt; e\n\n\n\nSynonyms:\nglv8/util.find_function4\nglv8/util find_function4","url":"glv8/util.html#find_function4"},{"doc":"glv8/util","title":"get","content":"pub fn get(o: JsObject, k: a) -&gt; Result(JsObject, Nil)\n\n\n\nSynonyms:\nglv8/util.get\nglv8/util get","url":"glv8/util.html#get"},{"doc":"glv8/util","title":"info","content":"pub fn info() -&gt; Json\n\n\n\nSynonyms:\nglv8/util.info\nglv8/util info","url":"glv8/util.html#info"},{"doc":"glv8/util","title":"memory_usage","content":"pub fn memory_usage() -&gt; Json\n\n\n\nSynonyms:\nglv8/util.memory_usage\nglv8/util memory_usage","url":"glv8/util.html#memory_usage"},{"doc":"glv8/util","title":"merge","content":"pub fn merge(o: JsObject, k: a, v: b) -&gt; JsObject\n\n\n\nSynonyms:\nglv8/util.merge\nglv8/util merge","url":"glv8/util.html#merge"},{"doc":"glv8/util","title":"name","content":"pub fn name(a: a) -&gt; String\n\n\n\nSynonyms:\nglv8/util.name\nglv8/util name","url":"glv8/util.html#name"},{"doc":"glv8/util","title":"object","content":"pub fn object() -&gt; JsObject\n\n\n\nSynonyms:\nglv8/util.object\nglv8/util object","url":"glv8/util.html#object"},{"doc":"glv8/util","title":"reset","content":"pub fn reset(context ctx: String) -&gt; Nil\n\n\n\nSynonyms:\nglv8/util.reset\nglv8/util reset","url":"glv8/util.html#reset"},{"doc":"glv8/util","title":"reset_all","content":"pub fn reset_all() -&gt; Nil\n\n\n\nSynonyms:\nglv8/util.reset_all\nglv8/util reset_all","url":"glv8/util.html#reset_all"},{"doc":"glv8/util","title":"run_script","content":"pub fn run_script(script s: String, file f: String) -&gt; Nil\n\n\n\nSynonyms:\nglv8/util.run_script\nglv8/util run_script","url":"glv8/util.html#run_script"},{"doc":"glv8/util","title":"to_dynamic","content":"pub fn to_dynamic(a: a) -&gt; Dynamic\n\n\n\nSynonyms:\nglv8/util.to_dynamic\nglv8/util to_dynamic","url":"glv8/util.html#to_dynamic"},{"doc":"glv8/util","title":"to_json","content":"pub fn to_json(a: a) -&gt; Json\n\n\n\nSynonyms:\nglv8/util.to_json\nglv8/util to_json","url":"glv8/util.html#to_json"},{"doc":"glv8/util","title":"version","content":"pub fn version() -&gt; String\n\n\n\nSynonyms:\nglv8/util.version\nglv8/util version","url":"glv8/util.html#version"},{"doc":"glv8/util","title":"glv8/util","content":"","url":"glv8/util.html"},{"doc":"app/plv8","title":"Rec","content":"pub type Rec {\n  Rec(i: Int, t: String)\n}\n\nRec(i: Int, t: String)\n\n\nSynonyms:\napp/plv8.Rec\napp/plv8 Rec","url":"app/plv8.html#Rec"},{"doc":"app/plv8","title":"catch_sql_error","content":"pub fn catch_sql_error() -&gt; Nil\n\nSynonyms:\napp/plv8.catch_sql_error\napp/plv8 catch_sql_error","url":"app/plv8.html#catch_sql_error"},{"doc":"app/plv8","title":"catch_sql_error2","content":"pub fn catch_sql_error2() -&gt; String\n\nSynonyms:\napp/plv8.catch_sql_error2\napp/plv8 catch_sql_error2","url":"app/plv8.html#catch_sql_error2"},{"doc":"app/plv8","title":"fastsum","content":"pub fn fastsum(arr: Array(Int)) -&gt; Int\n\nSynonyms:\napp/plv8.fastsum\napp/plv8 fastsum","url":"app/plv8.html#fastsum"},{"doc":"app/plv8","title":"plv8_test","content":"pub fn plv8_test(\n  keys ka: Array(String),\n  values va: Array(String),\n) -&gt; String\n\nSynonyms:\napp/plv8.plv8_test\napp/plv8 plv8_test","url":"app/plv8.html#plv8_test"},{"doc":"app/plv8","title":"return_sql","content":"pub fn return_sql() -&gt; Array(Rec)\n\nSynonyms:\napp/plv8.return_sql\napp/plv8 return_sql","url":"app/plv8.html#return_sql"},{"doc":"app/plv8","title":"scalar_to_record","content":"pub fn scalar_to_record(i: Int, t: String) -&gt; Rec\n\nSynonyms:\napp/plv8.scalar_to_record\napp/plv8 scalar_to_record","url":"app/plv8.html#scalar_to_record"},{"doc":"app/plv8","title":"set_of_integers","content":"pub fn set_of_integers() -&gt; Array(Int)\n\nSynonyms:\napp/plv8.set_of_integers\napp/plv8 set_of_integers","url":"app/plv8.html#set_of_integers"},{"doc":"app/plv8","title":"set_of_records","content":"pub fn set_of_records() -&gt; Array(Rec)\n\nSynonyms:\napp/plv8.set_of_records\napp/plv8 set_of_records","url":"app/plv8.html#set_of_records"},{"doc":"app/plv8","title":"app/plv8","content":" the module implements some plv8 examples\n","url":"app/plv8.html"},{"doc":"bundle","title":"exports","content":"pub fn exports() -&gt; JsObject\n\nSynonyms:\nbundle.exports\nbundle exports","url":"bundle.html#exports"},{"doc":"bundle","title":"sqls","content":"pub fn sqls() -&gt; List(String)\n\nSynonyms:\nbundle.sqls\nbundle sqls","url":"bundle.html#sqls"},{"doc":"bundle","title":"bundle","content":"","url":"bundle.html"},{"doc":"build","title":"bundle_build","content":"pub fn bundle_build(\n  entry e: String,\n  namespace n: String,\n  outfile o: String,\n) -&gt; Promise(Result(Nil, String))\n\nSynonyms:\nbuild.bundle_build\nbuild bundle_build","url":"build.html#bundle_build"},{"doc":"build","title":"bundle_watch","content":"pub fn bundle_watch(\n  entry e: String,\n  namespace n: String,\n  outfile o: String,\n) -&gt; Promise(Result(Nil, String))\n\nSynonyms:\nbuild.bundle_watch\nbuild bundle_watch","url":"build.html#bundle_watch"},{"doc":"build","title":"copy_build","content":"pub fn copy_build(\n  sql f: String,\n  namespace n: String,\n  outfile o: String,\n) -&gt; Promise(Result(Nil, String))\n\nSynonyms:\nbuild.copy_build\nbuild copy_build","url":"build.html#copy_build"},{"doc":"build","title":"copy_watch","content":"pub fn copy_watch(\n  sql f: String,\n  namespace n: String,\n  outfile o: String,\n) -&gt; Promise(Result(Nil, String))\n\nSynonyms:\nbuild.copy_watch\nbuild copy_watch","url":"build.html#copy_watch"},{"doc":"build","title":"main","content":"pub fn main() -&gt; Promise(Result(Nil, Nil))\n\nSynonyms:\nbuild.main\nbuild main","url":"build.html#main"},{"doc":"build","title":"build","content":"","url":"build.html"},{"doc":"glv8/window","title":"WindowObject","content":"pub type WindowObject\n\n\nSynonyms:\nglv8/window.WindowObject\nglv8/window WindowObject","url":"glv8/window.html#WindowObject"},{"doc":"glv8/window","title":"get_window_object","content":"pub fn get_window_object() -&gt; WindowObject\n\n\n\nSynonyms:\nglv8/window.get_window_object\nglv8/window get_window_object","url":"glv8/window.html#get_window_object"},{"doc":"glv8/window","title":"glv8/window","content":"","url":"glv8/window.html"}]);